@@ -14,6 +14,7 @@
 #include <utility>
 #include <tuple>
 #include <algorithm>
+#include <numeric>
 #include <iterator>
 
 #include "xexpression.hpp"
@@ -130,6 +131,7 @@ namespace xt
         template <class Func>
         xfunction(Func&& f, CT... e) noexcept;
 
+        size_type size() const noexcept;
         size_type dimension() const noexcept;
         const shape_type& shape() const;
 
@@ -137,6 +139,7 @@ namespace xt
         const_reference operator()(Args... args) const;
 
         const_reference operator[](const xindex& index) const;
+        const_reference operator[](size_type i) const;
 
         template <class It>
         const_reference element(It first, It last) const;
@@ -189,7 +192,8 @@ namespace xt
 
         std::tuple<CT...> m_e;
         functor_type m_f;
-        shape_type m_shape;
+        mutable shape_type m_shape;
+        mutable bool m_shape_computed;
 
         friend class xfunction_iterator<F, R, CT...>;
         friend class xfunction_stepper<F, R, CT...>;
@@ -262,7 +266,6 @@ namespace xt
         using pointer = typename xfunction_type::const_pointer;
         using size_type = typename xfunction_type::size_type;
         using difference_type = typename xfunction_type::difference_type;
-        using iterator_category = std::input_iterator_tag;
 
         using shape_type = typename xfunction_type::shape_type;
 
@@ -313,9 +316,9 @@ namespace xt
     template <class F, class R, class... CT>
     template <class Func>
     inline xfunction<F, R, CT...>::xfunction(Func&& f, CT... e) noexcept
-        : m_e(e...), m_f(std::forward<Func>(f)), m_shape(make_sequence<shape_type>(compute_dimension(), size_type(1)))
+        : m_e(e...), m_f(std::forward<Func>(f)), m_shape(make_sequence<shape_type>(0, size_type(1))),
+          m_shape_computed(false)
     {
-        broadcast_shape(m_shape);
     }
     //@}
 
@@ -323,13 +326,23 @@ namespace xt
      * @name Size and shape
      */
     //@{
+    /**
+     * Returns the size of the expression.
+     */
+    template <class F, class R, class... CT>
+    inline auto xfunction<F, R, CT...>::size() const noexcept -> size_type
+    {
+        return compute_size(shape());
+    }
+
     /**
      * Returns the number of dimensions of the function.
      */
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::dimension() const noexcept -> size_type
     {
-        return m_shape.size();
+        size_type dimension = m_shape_computed ? m_shape.size() : compute_dimension();
+        return dimension;
     }
 
     /**
@@ -338,6 +351,12 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::shape() const -> const shape_type&
     {
+        if (!m_shape_computed)
+        {
+            m_shape = make_sequence<shape_type>(compute_dimension(), size_type(1));
+            broadcast_shape(m_shape);
+            m_shape_computed = true;
+        }
         return m_shape;
     }
     //@}
@@ -363,7 +382,13 @@ namespace xt
     {
         return element(index.cbegin(), index.cend());
     }
-    
+ 
+    template <class F, class R, class... CT>
+    inline auto xfunction<F, R, CT...>::operator[](size_type i) const -> const_reference
+    {
+        return operator()(i);
+    }
+ 
     /**
      * Returns a constant reference to the element at the specified position in the function.
      * @param first iterator starting the sequence of indices
@@ -467,7 +492,7 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::xbegin() const noexcept -> const_broadcast_iterator
     {
-        return const_broadcast_iterator(stepper_begin(m_shape), &m_shape);
+        return const_broadcast_iterator(stepper_begin(shape()), &shape());
     }
 
     /**
@@ -477,7 +502,7 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::xend() const noexcept -> const_broadcast_iterator
     {
-        return const_broadcast_iterator(stepper_end(m_shape), &m_shape);
+        return const_broadcast_iterator(stepper_end(shape()), &shape());
     }
 
     /**
