@@ -40,7 +40,6 @@ namespace xt
     template <class CT, class M, std::size_t I>
     class xoffsetview;
 
-
     /*****************************
      * offsetview_temporary_type *
      *****************************/
@@ -125,12 +124,14 @@ namespace xt
         template <class E>
         disable_xexpression<E, self_type>& operator=(const E& e);
 
+        size_type size() const noexcept;
         size_type dimension() const noexcept;
         const shape_type & shape() const noexcept;
 
         template <class... Args>
         const_reference operator()(Args... args) const;
         const_reference operator[](const xindex& index) const;
+        const_reference operator[](size_type i) const;
 
         template <class It>
         const_reference element(It, It last) const;
@@ -355,6 +356,15 @@ namespace xt
     /**
      * @name Size and shape
      */
+    /**
+     * Returns the size of the expression.
+     */
+    template <class CT, class M, std::size_t I>
+    inline auto xoffsetview<CT, M, I>::size() const noexcept -> size_type
+    {
+        return m_e.size();
+    }
+
     /**
      * Returns the number of dimensions of the expression.
      */
@@ -401,7 +411,13 @@ namespace xt
     {
         return forward_offset<value_type, I>(m_e[index]);
     }
-    
+ 
+    template <class CT, class M, std::size_t I>
+    inline auto xoffsetview<CT, M, I>::operator[](size_type i) const -> const_reference
+    {
+        return operator()(i);
+    }
+ 
     /**
      * Returns a constant reference to the element at the specified position in the expression.
      * @param first iterator starting the sequence of indices
