@@ -27,10 +27,8 @@ namespace xt
 
     public:
 
-        // Yes, really. std::enable_if<true> as a workaround to MSVC madness.
-        // (buggy error C2210 when mixing parameter packs and template aliases)
         template <class... E>
-        using xfunction_type = std::enable_if_t<true, xfunction<F, R, xclosure_t<E>...>>;
+        using xfunction_type = xfunction<F, R, xclosure_t<E>...>;
 
         template <class Func>
         explicit xvectorizer(Func&& f);
